import os
from urllib.parse import urlparse, urlunparse, parse_qsl, parse_qs, urljoin


def abs_path(root, url):
    # print('abs_path:root', root)
    # print('abs_path:url', url)
    return urljoin(root, url)


def parse_path(url):
    p = urlparse(url)
    args = '', '', p.path, p.params, p.query, p.fragment
    return urlunparse(args)


def parse_root(url):
    p = urlparse(url)
    args = p.scheme, p.netloc, '', '', '', ''
    return urlunparse(args)


def parse_query(url, qsl=True):
    parse = urlparse(url)
    queryset = parse_qsl(parse.query) if qsl == True else parse_qs(parse.query)
    return dict(queryset)


def parse_src(url):
    p = urlparse(url)
    return os.path.basename(p.path)


def queryjoin(url, params=None):
    r = Request('GET', url, params=params)
    s = Session()
    url = s.prepare_request(r).url
    s.close()
    return url


def queryjoin(url, query=None):
    if query is None:
        return url
    qsl = ['{}={}'.format(qry, val) for qry, val in query.items()]
    qs = '&'.join(qsl)
    p = urlparse(url)
    args = p.scheme, p.netloc, p.path, p.params, qs, p.fragment
    return urlunparse(args)


def filter_params(url, fields):
    if fields is None:
        return url
    p = urlparse(url)
    query = parse_query(url)
    qsl = ['{}={}'.format(q, query.get(q, '')) for q in fields]
    qs = '&'.join(qsl)
    p = urlparse(url)
    args = p.scheme, p.netloc, p.path, p.params, qs, p.fragment
    return urlunparse(args)



import time

import requests
import requests_cache
from fake_useragent import FakeUserAgent

from scrapgo import settings
from scrapgo.lib.time.time import get_random_second
from scrapgo.utils.shortcuts import filter_params


class CachedRequests(object):
    USER_AGENT_NAME = settings.USER_AGENT_NAME  # default to 'chrome'
    HEADERS = None
    REQUEST_DELAY = settings.REQUEST_DELAY
    CACHE_NAME = settings.CACHE_NAME
    CACHE_BACKEND = settings.CACHE_BACKEND
    CACHE_EXPIRATION = settings.CACHE_EXPIRATION
    CACHE_METHODS = settings.CACHE_METHODS

    def __init__(self, **kwargs):
        super().__init__(**kwargs)
        self.headers = {
            'User-Agent': getattr(FakeUserAgent(), self.USER_AGENT_NAME)
        }
        if self.HEADERS:
            self.headers.update(self.HEADERS)

        self.requests = requests_cache.CachedSession(
            cache_name=self.CACHE_NAME,
            backend=self.CACHE_BACKEND,
            expire_after=self.CACHE_EXPIRATION,
            allowable_methods=self.CACHE_METHODS
        )

    def _refresh_cache(self, url):
        if self.requests.cache.has_url(url):
            self.requests.cache.delete_url(url)

    def _delay_control(self, response):
        if response.from_cache == False:
            delay = self._get_delay()
            time.sleep(delay)
            log = 'get {} from cache=={} (delay:{}s)'
            print(log.format(response.url, response.from_cache, delay))
        else:
            log = 'get {} from_cache=={}'
            print(log.format(response.url, response.from_cache))

    def _get_delay(self):
        if isinstance(self.REQUEST_DELAY, (tuple, list,)) and len(self.REQUEST_DELAY) == 2:
            return get_random_second(*self.REQUEST_DELAY)
        return self.REQUEST_DELAY

    def _validate_response(self, response):
        if not response.content:
            self.requests.cache.delete_url(url)
            print('Warning: {} has no content!'.format(response.url))

    def _get(self, url, headers=None, refresh=False, fields=None):
        headers = headers or self.get_header()
        url = filter_params(url, fields)
        if refresh:
            self._refresh_cache(url)

        r = self.requests.get(url, headers=headers)
        r.raise_for_status()
        self._delay_control(r)
        self._validate_response(r)
        return r

    def _post(self, url, data, headers=None, refresh=True, **kwargs):
        headers = headers or self.headers
        if refresh:
            self._refresh_cache(url)
        r = self.requests.post(url, data=data, headers=headers, **kwargs)
        print('post {}'.format(url))
        r.raise_for_status()
        return r

    def get_header(self):
        return dict(self.headers)

    def set_header(self, headers):
        self.headers = headers



import re
import os
import hashlib
from datetime import datetime
from urllib.parse import quote, urljoin, unquote
from collections import OrderedDict
from pprint import pprint


from .shortcuts import hexMD5, get_public_ip

from scrapgo import LinkRelayScraper, url, urlpattern
from scrapgo.utils import mkdir_p, cp, parse_query, queryjoin, parse_root


class DrugInfoScraper(LinkRelayScraper):
    LOGIN_URL = 'https://www.druginfo.co.kr/login/login.aspx'
    ROOT_URL = 'https://www.druginfo.co.kr/search2/search.aspx'
    CACHE_NAME = 'druginfo'

    HEADERS = {
        'Content-Type': 'application/x-www-form-urlencoded',
        'Host': 'www.druginfo.co.kr',
        'Origin': 'https://www.druginfo.co.kr',
        'Referer': 'https://www.druginfo.co.kr/',
    }

    LINK_RELAY = [
        urlpattern(
            r'^/detail/product.aspx\?pid=(?P<pid>.+)$',
            parser='detail_parser', refresh=True
        )
    ]

    def _gen_login_data(self, id, pw, ip):
        timestamp = datetime.now().strftime("%Y%m%d%H")
        return {
            'id': id,
            'ip': ip,
            't_passwd': pw,
            'passwd': hexMD5(timestamp+hexMD5(pw)+ip),
            'timestamp': timestamp
        }

    def login(self, id, passwd):
        ip = get_public_ip()
        print('ip', ip)
        login_data = self._gen_login_data(id, passwd, ip)
        super().login(login_data)

    def detail_parser(self, response, **kwargs):
        match = response.scrap.match
        soup = response.scrap.soup
        print('detail_parser:pid', match('pid'))
        title = re.sub('\s+', ' ', soup.title.text).strip()
        print(title)


def drug_search(params):
    params['q'] = quote(params['q'], encoding='cp949')
    di = DrugInfoScraper(root_params=params)
    print(di.LINK_RELAY[0].refresh)
    di.login('anonymous04', 'admindg04!')
    di.scrap()



import hashlib
import re
import os
import sys
import platform
from collections import OrderedDict

from requests import get, Session


def ip42pl():
    return get('http://ip.42.pl/raw').text


def jsonip():
    return get('http://jsonip.com').json()['ip']


def httpbin():
    return get('http://httpbin.org/ip').json()['origin'].split(',')[0]


def ipify():
    return get('https://api.ipify.org/?format=json').json()['ip']


def get_public_ip():
    for f in [httpbin, ip42pl, jsonip, ipify]:
        try:
            ret = f()
        except Exception as e:
            pass
        else:
            return ret


def hexMD5(value):
    h = hashlib.md5()
    h.update(value.encode())
    return h.hexdigest()


def read_keyword_file(_file):
    if not _file:
        return
    with open(_file) as fp:
        keyword_list = fp.readlines()
        keyword_list = list(map(str.strip, keyword_list))
        return keyword_list


def _float2str(float_val):
    try:
        val = str(int(float_val))
    except Exception as e:
        return float_val
    else:
        return val



from collections import UserDict, namedtuple

History = namedtuple('History', ['url', 'previous', 'name'])


class HistoryDict(UserDict):

    def set_history(self, url, previous, name=None):
        if url in self:
            print('warning: {} has already exist in history'.format(url))
        self[url] = History(url, previous, name)

    def get_tracer(self, url):
        tracer = []
        previous = url
        while previous:
            try:
                p = self[previous]
                previous = p.previous
            except:
                return tracer
            else:
                if p.previous:
                    tracer.insert(0, p)
        return tracer

    def get_previous(self, url, name=None):
        history = self.get(url)
        if not history:
            return
        if name is None:
            return history.previous

        for history in self.get_tracer(url):
            if history.name == name:
                return history.url

    def get_histories(self, name):
        return [h.url for h in self.values() if h.name == name]



from scrapgo.modules import RequestsManager


test_url1 = 'https://comic.naver.com/webtoon/list.nhn?titleId=703846&weekday=tue'
test_url2 = 'https://image-comic.pstatic.net/webtoon/703846/48/20190225210901_bb6b8e3cf2b04f797165a7ba7eefb171_IMAG01_1.jpg'

test = RequestsManager()

r = test._get(test_url1)
test._get(test_url2)
print(r.content[:100])



from scrapgo.scraper import LinkRelayScraper

import re

REGX_TOON_HREF = re.compile(
    r'^/webtoon/list.nhn\?titleId=(?P<titleId>\d+)&weekday=(?P<weekday>\w*)&page=(?P<page>\d+)$')


class NaverWebToonScraper(LinkRelayScraper):
    ROOT_URL = 'https://comic.naver.com/webtoon/weekday.nhn'

    def main_filter(self, link, match, query, context):
        p = match('page')
        return int(p) < 10


nw = NaverWebToonScraper()
r'https://comic.naver.com/webtoon/list.nhn?titleId=642653&weekday=tue&page={page}'
res = nw.get(
    'https://comic.naver.com/webtoon/list.nhn?titleId=626907&weekday=wed&page=1')
responses = nw._crawl_link_pattern(res, REGX_TOON_HREF, recursive=True)
# for r in responses:
# print(r.url)
list(responses)



# import pandas as pd
# import os
# import re
# import json
# from io import BytesIO
# from collections import namedtuple, OrderedDict

# from scrapgo.scraper import LinkRelayScraper, url, urlpattern
# from scrapgo.utils.shortcuts import mkdir_p, cp, parse_query, queryjoin, parse_root


# def _get_td(tr, index, parse=True):
#     tds = tr('td')
#     if tds:
#         td = tds[index]
#         if parse:
#             if td and td.text:
#                 return td.text.strip()
#         else:
#             return td


# class Iam176ReviewScraper(LinkRelayScraper):
#     ROOT_URL = 'http://www.iam176.co.kr/board/board.html?code=iam176_board2'
#     REQUEST_DELAY = 0, 1,
#     CACHE_NAME = 'IMCACHE'
#     HEADERS = {
#         'Cookie': '____MSLOG__initkey=0.9432849585157661; db=JyEqM54nxlN98ZnjMQ3N6zoMZuvLw7GtAKuJZgdGxNGS%2F3LFcIXvC5hoLbWUDMu0EuVgny5z1i2HvICTHmMAEmqmDHsfsVOonjX7awQtV0cYHOPxidvVY%2Bxn3CGkheRV; shop_language=kor; MakeshopLogUniqueId=667d8c7b905e7a1ebd99ef3e7c47eb09; ____MSLOG__initday=20190324; _Rk_K_eJ1=%22.%22; member_param=type%3Dboard%26code%3Diam176_image1; MSECURESESSION=56bc6de39559f6cf1610a1a1635944dae0954ff70aacf778a44a3f9b6373692c; viewproduct=%2C003000001279%2C003000001299%2C003000001294%2C; _Rk_K_cA2=%7B%22branduid%22%3A%22344473%22%7D; wcs_bt=s_1a81591b2e88:1553425818'
#     }
#     LINK_RELAY = [
#         urltemplate(
#             'http://www.iam176.co.kr/board/board.html?code=iam176_board2&page={page}&board_cate=#board_list_target',
#             renderer='list_url_renderer',
#             parser='list_parser',
#             name='list',
#             refresh=True
#         ),
#         urlpattern(
#             r'^/board/board.html\?code=iam176_board2&page=(?P<page>\d+)&type=(?P<type>\w+)&board_cate=&num1=(?P<num1>\d+)&num2=(?P<num2>\d+)&number=(?P<number>\d+)&lock=(?P<lock>\w+)$',
#             parser='detail_parser',
#             name='detail',
#             fields=['code', 'type', 'num1', 'num2', 'number']
#         ),
#         urlpattern(
#             r'^http://board.makeshop.co.kr/board/premium183/iam176_board2/(?P<filename>.+)$',
#             parser='image_parser',
#             name='image',
#             referer='detail',
#             refresh=True
#         )
#     ]

#     def list_url_renderer(self, template, context=None):
#         start = context['start']
#         end = context['end']
#         for page in range(start, end+1):
#             yield template.format(page=page)

#     def list_parser(self, response, match, soup, context):
#         table = soup.find('table', summary="No, content,Name,Data,Hits")
#         if table is None:
#             print('list_parser', response.url)
#             return

#         def parse_no(tr):
#             td = _get_td(tr, 0)
#             return td

#         def parse_title(tr):
#             td = _get_td(tr, 3)
#             return td.replace('\n', ' ')

#         def parse_author(tr):
#             td = _get_td(tr, 4, parse=False)
#             authors = td('div', class_='video-writer')
#             if authors:
#                 return authors[0].text.replace('\n', ' ').replace('회원게시글검색', '').strip()
#             else:
#                 return 'Unknown'

#         def parse_date(tr):
#             td = _get_td(tr, 5)
#             return td.strip().replace('/', '-')

#         for tr in table('tr'):
#             no = parse_no(tr)
#             if no is None:
#                 continue
#             yield {
#                 'no': no,
#                 'title': parse_title(tr),
#                 'author': parse_author(tr),
#                 'published': parse_date(tr),
#             }

#     def detail_parser(self, response, match, soup, context):
#         content_area = soup('div', class_='data-bd-cont')[0]
#         content = content_area.text.strip()
#         no = match('number')
#         return {
#             'content': content,
#             'no': no,
#             'href': response.url
#         }

#     def image_parser(self, response, match, soup, context):
#         return {
#             'referer': response.previous,
#             'src': response.url
#         }


# def review(context):
#     filname = 'results({start}~{end}).csv'.format(**context)
#     save_path = os.path.join(context['save_to'], filname)
#     iam = Iam176ReviewScraper()
#     r = iam.scrap(context=context)
#     df_list = pd.DataFrame(r['list'])
#     df_detail = pd.DataFrame(r['detail'])
#     df_image = pd.DataFrame(r['image'])
#     df = pd.merge(df_list, df_detail)
#     df = pd.merge(df, df_image, left_on='href', right_on='referer')

#     df.to_csv(save_path)



from .urlparser import *
import json


def mkdir_p(path):
    if not os.path.isdir(path):
        os.makedirs(path, exist_ok=True)


def cp(path, content):
    with open(path, 'wb') as fp:
        fp.write(content)


def parse_jsonp(jsonp, **kwargs):
    js = jsonp[jsonp.index("(") + 1: jsonp.rindex(")")]
    return json.loads(js, **kwargs)



import re
from collections import namedtuple

from bs4 import BeautifulSoup

from scrapgo.utils.shortcuts import parse_src
from scrapgo import settings


def is_parsable(response, allowed_content_types=settings.PARSE_CONTENT_TYPES):
    content_type_info = response.headers.get('Content-Type')
    if content_type_info:
        for ctype in allowed_content_types:
            if ctype in content_type_info:
                return True
    return False


def prettify_text(text):
    lines = []
    for line in text.split('\n'):
        txt = re.sub('\s+', ' ', line).strip()
        lines.append(txt)
    lines = filter(None, lines)
    return '\n'.join(lines)


def prettify_textarea(soup):
    linebreakers = ['a', 'p', 'div', 'h3', 'br']
    if soup:
        for line in soup(linebreakers):
            line.replace_with(line.text + '\n')
        if soup.text:
            content = prettify_text(soup.text)
            return content
    return ''


class SoupParserMixin(object):
    CRAWL_TARGET_ATTRS = settings.CRAWL_TARGET_ATTRS
    PARSE_CONTENT_TYPES = settings.PARSE_CONTENT_TYPES

    def __init__(self, parser=settings.BEAUTIFULSOUP_PARSER, **kwargs):
        super().__init__(**kwargs)
        self.parser = parser

    def _is_parsable(self, response):
        content_type_info = response.headers.get('Content-Type')
        if not content_type_info:
            return False
        for ctype in self.PARSE_CONTENT_TYPES:
            if ctype in content_type_info:
                return True
        return False

    def _get_soup(self, response):
        if self._is_parsable(response):
            content = response.content
            return BeautifulSoup(content, self.parser)
        return BeautifulSoup('', self.parser)

    def _parse_link(self, response, link_pattern):
        soup = self._get_soup(response)
        parsed = set()

        for attr in self.CRAWL_TARGET_ATTRS:
            for link in soup('', {attr: link_pattern}):
                if link not in parsed:
                    parsed.add(link[attr])
                    yield link[attr]

    def find_texts(self, soup, *args, **kwargs):
        ret = []
        for tag in soup(*args, **kwargs):
            text = tag.text or ''
            ret.append(text.strip())
        return ret

    def prettify_textarea(self, soup):
        linebreakers = ['a', 'p', 'div', 'h3', 'br']
        if soup:
            for line in soup(linebreakers):
                line.replace_with(line.text + '\n')
            if soup.text:
                content = prettify_text(soup.text)
                return content
        return ''



import os
import re
import json
import math
from io import BytesIO
from collections import namedtuple, OrderedDict
import pandas as pd

from scrapgo import LinkRelayScraper, url, urlpattern, root
from scrapgo.utils import mkdir_p, cp, parse_query, queryjoin, parse_root, parse_jsonp


COMMENT_PAGE_SIZE = 100


class NaverWebToonScraper(LinkRelayScraper):
    ROOT_URL = 'https://comic.naver.com/webtoon/weekday.nhn'
    REQUEST_DELAY = 0
    LINK_RELAY = [
        urlpattern(
            r'^/webtoon/list.nhn\?titleId=(?P<titleId>\d+)&weekday=(?P<weekday>\w+)$',
            parser='toon_parser',
            name='toon',
        ),
        urlpattern(  # 이미지, 파일등 과같은 source 항목을 다운로드 필터링등 처리할 수 있다
            r'^https://shared-comic.pstatic.net/thumb/webtoon/(?P<titleId>\d+)/thumbnail/(?P<filename>.+)$',
            parser='toon_thumb_parser',
            referer='toon',
            # refresh=True,
            name='toon_thumb',
        ),
        url(
            'https://comic.naver.com/webtoon/list.nhn?titleId={titleId}&weekday=tue&page={page}',
            generator='toon_page_urlrenderer',
            name='toon_page',
        ),
        urlpattern(
            r'^https://shared-comic.pstatic.net/thumb/webtoon/(?P<titleId>\d+)/(?P<no>\d+)/(?P<filename>.+)$',
            parser='episode_thumb_parser',
            name='episode_thumb',
            referer='toon_page',
        ),
        urlpattern(
            r'^/webtoon/detail.nhn\?titleId=(?P<titleId>\d+)&no=(?P<no>\d+)&weekday=(?P<weekday>\w*)$',
            parser='episode_parser',
            name='episode'
        ),
        urlpattern(
            r'^https://image-comic.pstatic.net/webtoon/(?P<titleId>\d+)/(?P<no>\d+)/(?P<filename>.+)$',
            parser='episode_cut_parser',
            referer='episode',
            name='episode_cut',
        ),
        url(
            'https://apis.naver.com/commentBox/cbox/web_naver_list_jsonp.json',
            generator='comment_urlrenderer',
            breaker='comment_page_breaker',
            parser='comment_parser',
            referer='episode',
            refresh=True,
        )
    ]

    def main_filter(self, link, query, match, context):
        if not match:
            return True
        titleId = context['titleId']
        return titleId and titleId == match('titleId')

    def toon_parser(self, response, context):
        match = response.scrap.match
        soup = response.scrap.soup

        def get_title(soup):
            titlebar = soup.title.text.split('::')
            title = titlebar[0].strip()
            return title

        def get_author(soup):
            author = soup.select_one('div.detail .wrt_nm')
            text = self.prettify_textarea(author)
            return text

        def get_description(soup):
            description = soup.select_one('div.detail > p')
            text = self.prettify_textarea(description)
            return text

        toon = dict(
            titleId=match('titleId'),
            title=get_title(soup),
            author=get_author(soup),
            description=get_description(soup)
        )
        return toon

    def toon_thumb_parser(self, response, context):
        toon_thumb = dict(
            titleId=response.scrap.match('titleId'),
            toon_thumb_src=response.url,
            # toon_thumb_content=response.content
        )
        return toon_thumb

    def toon_page_urlrenderer(self, parent_response, template, context):
        soup = parent_response.scrap.soup
        match = parent_response.scrap.match

        def get_page_range(soup):
            action = self.get_action('episode')
            pattern = action.regex
            no_set = set()
            for a in soup('a', href=pattern):
                href = a['href']
                match = pattern.match(href)
                no = int(match.group('no'))
                no_set.add(no)
            page_count = max(no_set) // len(no_set) + 1
            page_range = range(1, page_count + 1)
            return page_range

        page_range = get_page_range(soup)

        for page in page_range:
            yield template.format(titleId=context['titleId'], page=page)

    def episode_thumb_parser(self, response, context):
        episode_thumb = dict(
            titleId=response.scrap.match('titleId'),
            no=response.scrap.match('no'),
            episode_thumb_src=response.url
        )
        return episode_thumb

    def episode_parser(self, response, context):
        soup = response.scrap.soup
        match = response.scrap.match

        def get_episode_title(soup):
            title = soup.select_one('div.view > h3')
            text = title.text.strip()
            return text

        episode = dict(
            titleId=match('titleId'),
            no=match('no'),
            episode_title=get_episode_title(soup)
        )
        context['comment_kwargs'] = {
            'titleId': match('titleId'),
            'no': match('no')
        }
        return episode

    def episode_cut_parser(self, response, context):
        match = response.scrap.match
        episode_cut = dict(
            titleId=match('titleId'),
            no=match('no'),
            episode_cut_src=response.url
        )
        return episode_cut

    def _get_comment_url(self, path, titleId, no, page=1):
        params = dict(
            ticket='comic',
            templateId='webtoon',
            pool='cbox3',
            lang='ko',
            country='KR',
            objectId='{}_{}'.format(titleId, no),
            pageSize=COMMENT_PAGE_SIZE,
            indexSize='10',
            listType='OBJECT',
            page=page,
            refresh='false',
            sort='NEW'
        )
        url = queryjoin(path, params)
        return url

    def comment_urlrenderer(self, parent_response, path, context):
        match = parent_response.scrap.match
        titleId = match('titleId')
        no = match('no')
        for page in range(1, 100000):
            url = self._get_comment_url(path, titleId, no, page)
            yield url

    def comment_page_breaker(self, response, context):
        query = response.scrap.query
        json = parse_jsonp(response.text)
        total_comment_count = json['result']['count']['total']
        end_page = math.ceil(total_comment_count/COMMENT_PAGE_SIZE)
        page = int(query['page'])
        return page == end_page

    def comment_parser(self, response, context):
        titleId = context['comment_kwargs']['titleId']
        no = context['comment_kwargs']['no']
        json = parse_jsonp(response.text)
        comment_list = json['result']['commentList']
        for comment in comment_list:
            cmt = dict(
                titleId=titleId,
                no=no,
                comment_content=comment['contents'],
                comment_no=comment['commentNo']
            )
            comment['titleId'] = titleId
            comment['no'] = no
            yield comment


def retrive_webtoon(context):
    comment_save_to = os.path.join(context['save_to'], 'comment.csv')
    n = NaverWebToonScraper()
    r = n.scrap(context=context)
    toon = pd.DataFrame(r['toon'])
    toon_thumb = pd.DataFrame(r['toon_thumb'])
    episode_thumb = pd.DataFrame(r['episode_thumb'])
    episode = pd.DataFrame(r['episode'])
    episode_cut = pd.DataFrame(r['episode_cut'])
    comment = pd.DataFrame(r['comment'])

    # df_toon = pd.merge(toon, toon_thumb)
    # df_episode = pd.merge(episode, episode_thumb, on=['titleId', 'no'])
    # df_episode = pd.merge(df_episode, episode_cut, on=['titleId', 'no'])
    # df = pd.merge(df_toon, df_episode)
    # df.to_csv(comment_save_to)
    # comment.to_csv(comment_save_to)



import re
from collections import namedtuple
import math


class BaseAction(object):

    def __init__(self, name=None, fields=None, filter=None, breaker=None, parser=None, static=False, refresh=False, referer=None):
        self.name = name
        self.fields = fields
        self.filter = filter
        self.breaker = breaker
        self.parser = parser
        self.static = static
        self.refresh = refresh
        self.referer = referer


class Root(BaseAction):
    def __init__(self, url, **kwargs):
        super().__init__(**kwargs)
        self.name = self.name or url
        self.url = url


class Url(BaseAction):
    def __init__(self, url, generator=None, previewer=None, **kwargs):
        super().__init__(**kwargs)
        self.url = url
        self.name = self.name or url
        self.previewer = previewer
        self.generator = generator


class RegexUrl(BaseAction):
    def __init__(self, regex, recursive=False, **kwargs):
        super().__init__(**kwargs)
        self.name = self.name or regex
        self.regex = re.compile(regex)
        self.recursive = recursive


def root(url, fields=None, name=None, filter=None, breaker=None, parser=None, refresh=False, relay=True, referer=None):
    return Root(
        url,
        name=name,
        fields=fields,
        filter=filter,
        breaker=None,
        parser=parser,
        refresh=refresh,
        static=not relay,
        referer=referer
    )


def url(url, generator=None, previewer=None, fields=None, name=None, filter=None, breaker=None, parser=None, refresh=False, relay=True, referer=None):
    return Url(
        url,
        generator=generator,
        previewer=previewer,
        name=name,
        fields=fields,
        filter=filter,
        breaker=breaker,
        parser=parser,
        refresh=refresh,
        static=not relay,
        referer=referer
    )


def urlpattern(regx, fields=None, name=None, filter=None, breaker=None, parser=None, recursive=False, refresh=False, relay=True, referer=None):
    return RegexUrl(
        regx,
        name=name,
        fields=fields,
        filter=filter,
        breaker=breaker,
        parser=parser,
        recursive=recursive,
        refresh=refresh,
        static=not relay,
        referer=referer
    )



import os
import re
import json
import math
from pprint import pprint
from urllib.parse import unquote

import pandas as pd

from scrapgo import LinkRelayScraper, urlpattern, url, root
from scrapgo import settings
from scrapgo.utils import mkdir_p, cp, queryjoin


MAX_RESULT_PAGE = 150
COMMENT_PAGE_COUNT = 5


class NaverKinScraper(LinkRelayScraper):
    ROOT_URL = 'https://kin.naver.com/search/list.nhn'
    CRAWL_TARGET_ATTRS = settings.CRAWL_TARGET_ATTRS + ('content', )

    LINK_RELAY = [
        url(
            '/search/list.nhn',
            generator='question_page_urlrenderer',
            fields=['query', 'page'],
            name='results'
        ),
        urlpattern(
            r'^https://kin.naver.com/qna/detail.nhn\?d1id=(?P<d1id>\d+)&dirId=(?P<dirId>\d+)&docId=(?P<docId>\d+).+$',
            fields=['d1id', 'dirId', 'docId'],
            parser='question_parser',
            name='question'
        ),
        urlpattern(
            r'^https://kin-phinf.pstatic.net/(?P<date>.+)/.+/(?P<filename>.+)\?type=(?P<type>.+)$',
            parser='image_parser',
            referer='question',
            name='image',
        ),
        url(
            'https://kin.naver.com/ajax/detail/commentListAjax.nhn',
            fields=['answerNo', 'count', 'dirId', 'docId', 'page'],
            generator='comment_urlrenderer',
            referer='question',
            parser='comment_parser',
            refresh=True,
            name='comment',
        )
    ]

    def question_page_urlrenderer(self, parent_response, path, **kwargs):
        soup = parent_response.scrap.soup
        query = parent_response.scrap.query

        def get_result_count(soup):
            soup_count = soup.select_one('.number > em')
            count = 0
            if soup_count:
                count_text = soup_count.text
                count_text = count_text.replace(',', '')
                step, count = count_text.split('/')
                count = int(count)
            return count

        def get_page_size(soup):
            action = self.get_action('question')
            links = set(a['href'] for a in soup(href=action.regex))
            link_count = len(links)
            return link_count

        result_count = get_result_count(soup)
        page_size = get_page_size(soup)
        end_page = math.ceil(result_count/page_size) + 1
        end_page = min(MAX_RESULT_PAGE, end_page)
        for page in range(1, end_page+1):
            yield dict(
                query=query['query'],
                page=page
            )

    def question_parser(self, response, context, answer_list=None):
        soup = response.scrap.soup
        match = response.scrap.match

        def get_question_title(soup):
            title = soup.select_one('div.title')
            text = self.prettify_textarea(title)
            return text

        def get_question_content(soup):
            content_soup = soup.select_one('div.c-heading__content')
            content = self.prettify_textarea(content_soup)
            return content

        def get_answer_list(soup):
            answers = [{'answer_no': 0, 'answer_content': ''}]
            prefix = 'answer_'
            answer_areas = soup.select('div[id^={}]'.format(prefix))
            for answer_area in answer_areas:
                no = answer_area['id'].replace(prefix, '')
                answer_content = soup.select_one(
                    'div.c-heading-answer__content-user')
                content = self.prettify_textarea(answer_content)
                answer = dict(
                    answer_no=no,
                    answer_content=content
                )
                answers.append(answer)
            return answers

        question = dict(
            dirId=match('dirId'),
            docId=match('docId'),
            question_url=response.url,
            question_title=get_question_title(soup),
            question_content=get_question_content(soup)
        )

        answer_list = get_answer_list(soup)

        for answer in answer_list:
            answer.update(question)

        return answer_list

    def image_parser(self, response, context):
        print('image_parser:')

    def comment_urlrenderer(self, parent_response, path, context):
        query = parent_response.scrap.query
        dirId = query['dirId']
        docId = query['docId']

        soup = parent_response.scrap.soup
        prefix = 'cmtstr_'
        selector = 'a[id^={}]'.format(prefix)
        soup_comment_buttons = soup.select(selector)
        answer_no_list = []
        for soup_button in soup_comment_buttons:
            button_id = soup_button['id']
            answer_no = button_id.replace(prefix, '')
            comment_counter = soup_button.select_one('.button_compose_count')
            if comment_counter:
                count = comment_counter.text.strip()
                if count:
                    count = int(count)
                    end_page = math.ceil(count/COMMENT_PAGE_COUNT) + 1
                    page_range = range(1, end_page+1)
                    for page in page_range:
                        yield dict(
                            answerNo=answer_no,
                            count=COMMENT_PAGE_COUNT,
                            dirId=dirId,
                            docId=docId,
                            page=page
                        )

    def comment_parser(self, response, context):
        print('comment_parser:', response.url)
        comment_list = response.json()['result']['commentList']
        return comment_list


def naver_kin_with_image(params, context):
    nk = NaverKinScraper(root_params=params)
    r = nk.scrap(context=context)
    for row in r['question']:
        print(row)
        break
    for row in r['comment']:
        print(row)
        break



from practice.naver_kin_scraper.kin import naver_kin_with_image
import argparse
from practice.naver_webtoon_scraper.webtoon2 import retrive_webtoon
from practice.durginfo.druginfo_scraper import drug_search
import os
import sys

sys.path.append('.')


MEDIA_ROOT = 'media'


def main():
    argparser = argparse.ArgumentParser(
        formatter_class=argparse.ArgumentDefaultsHelpFormatter, description="TEST Scraper"
    )
    argparser.add_argument('keywords', help='검색할 키워드들 나열', nargs='*')
    argparser.add_argument('-titleId', '--titleId')

    argparser.add_argument('-search', '--search', nargs='?')

    argparser.add_argument('-start', '--start', type=int, default=1)
    argparser.add_argument('-end', '--end', type=int, default=1)

    args = argparser.parse_args()
    try:
        app = args.keywords[0]
    except:
        return

    context = {
        'save_to': MEDIA_ROOT
    }

    if app in ['webtoon', 'toon']:
        context['titleId'] = args.titleId
        retrive_webtoon(context)

    if app in ['kin']:
        context['save_to'] = os.path.join(MEDIA_ROOT, 'kin')
        naver_kin_with_image(params={'query': args.search}, context=context)

    if app in ['druginfo', 'di']:
        params = {
            'q': args.search
        }
        drug_search(params)


if __name__ == "__main__":
    main()



from scrapgo.modules import SoupParserMixin, RequestsManager


test_url1 = 'https://comic.naver.com/webtoon/weekday.nhn'

test_patterns = [
    r'^/webtoon/list.nhn\?titleId=(?P<titleId>\d+)&weekday=(?P<weekday>\w*)$',
    r'^https://shared-comic.pstatic.net/thumb/webtoon/(?P<titleId>\d+)/thumbnail/(?P<filename>.+)$'
]

rm = RequestsManager()

requests = rm._get(test_url1)

content = requests.content

test = SoupParserMixin()

soup = test._get_soup(requests)

r = test._collect_link(soup, test_patterns)

for link in r:
    print('collected_link:', link)
    # break



import os
from datetime import timedelta

BASE_DIR = os.path.dirname(os.path.dirname(os.path.abspath(__file__)))

# Request
USER_AGENT_NAME = 'chrome'
REQUEST_DELAY = 0.5

CACHE_NAME = 'REQUEST_CACHE'
CACHE_BACKEND = 'sqlite'
CACHE_EXPIRATION = timedelta(days=100)
CACHE_METHODS = 'GET',

# SoupParser
BEAUTIFULSOUP_PARSER = 'html.parser'
CRAWL_TARGET_ATTRS = 'href', 'src',
PARSE_CONTENT_TYPES = [
    'text/css', 'text/html', 'text/javascript', 'text/plain', 'text/xml'
]

# pattern context field names
URLPATTERN_DISPATCHER = 'urlpattern'
REDUCER_DISPATCHER = 'reducer'
URLFILTER_DISPATCHER = 'urlfilter'
REDUCER_NAMESPACE = 'namespace'



from collections import abc, defaultdict, deque, namedtuple
from itertools import chain
import re

from scrapgo.utils.shortcuts import *
from scrapgo.modules import CachedRequests, SoupParserMixin
from scrapgo.lib.history import HistoryDict
from .action.actions import Root, Url


ResponseMeta = namedtuple(
    'ResponseMeta', ['match', 'query', 'soup', 'history', 'context'])


class LinkRelayScraper(SoupParserMixin, CachedRequests):
    ROOT_URL = None
    LOGIN_URL = None
    LINK_RELAY = None

    def __init__(self, url=None, root_params=None, **kwargs):
        super().__init__(**kwargs)
        self.ROOT_URL = queryjoin(url or self.ROOT_URL, root_params)
        self.history = HistoryDict()
        self.LINK_RELAY = self.LINK_RELAY or []

    def _set_history(self, url, previous, name):
        url = abs_path(self.ROOT_URL, url)
        self.history.set_history(url, previous, name)

    def _parse_query_match(self, link, pattern=None):
        match = None
        query = parse_query(link)
        if pattern is not None and isinstance(pattern, re.Pattern):
            match = pattern.match(link).group
        return match, query

    def _set_response_meta(self, response, link, pattern=None, parent_response=None):
        match, query = self._parse_query_match(link, pattern)
        soup = self._get_soup(response)
        if parent_response is not None:
            context = parent_response.context
        else:
            context = {}
        meta = ResponseMeta(match, query, soup, self.history, context)
        setattr(response, 'scrap', meta)

    def _link_filter(self, link, pattern=None, filter=None, **kwargs):
        match, query = self._parse_query_match(link, pattern)
        if not self.main_filter(link, query, match, **kwargs):
            return False
        if filter:
            if not filter(link, query, match, **kwargs):
                return False
        return True

    def _set_referer(self, link, referer=None):
        header = self.get_header()
        if referer:
            previous = self.history.get_previous(link, referer)
            header['Referer'] = previous
        return header

    def _get_method(self, func, kind):
        if callable(func):
            return func
        if isinstance(func, str):
            if hasattr(self, func):
                return getattr(self, func)

        return {
            'filter': self.main_filter,
            'parser': self.default_parser,
            'previewer': self.url_previewer,
            'generator': self.query_generator,
            'breaker': self.crawl_breaker
        }[kind]

    def _crawl_link_pattern(self, parent_response, action, results, **kwargs):
        visited = set()
        responses = deque([parent_response])
        parent_url = parent_response.url
        parser = self._get_method(action.parser, 'parser')
        breaker = self._get_method(action.breaker, 'breaker')
        isbreak = False
        while responses:
            if isbreak:
                break
            r = responses.pop()
            for link in self._parse_link(r, action.regex):
                if link not in visited:
                    visited.add(link)
                    filtered = self._link_filter(
                        link=link,
                        pattern=action.regex,
                        filter=action.filter,
                        **kwargs
                    )
                    if not filtered:
                        continue
                    self._set_history(link, parent_url, action.name)
                    headers = self._set_referer(link, action.referer)
                    res = self.get(
                        link=link,
                        headers=headers,
                        refresh=action.refresh,
                        fields=action.fields
                    )
                    self._set_response_meta(res, link, action.regex)
                    parsed = parser(res, **kwargs)
                    self.reducer(parsed, action.name, results)
                    if action.recursive == True:
                        responses.append(res)
                    if self._is_parsable(res):
                        yield res
                    else:
                        if action.static == False:
                            action.static = True
                    isbreak = breaker(res, **kwargs)
                    if isbreak:
                        break

        if action.static == True:
            yield parent_response

    def _visited_links(self, links, parent_response, action, results, **kwargs):
        if isinstance(links, (str, abc.Mapping)):
            links = [links]

        if parent_response is None:
            parent_url = None
        else:
            parent_url = parent_response.url

        parser = self._get_method(action.parser, 'parser')
        breaker = self._get_method(action.breaker, 'breaker')

        for link in links:
            if isinstance(link, (abc.Mapping)):
                link = queryjoin(action.url, link)
            if self._link_filter(link, **kwargs):
                self._set_history(link, parent_url, action.name)
                headers = self._set_referer(link, action.referer)
                res = self.get(
                    link=link,
                    headers=headers,
                    refresh=action.refresh,
                    fields=action.fields
                )
                self._set_response_meta(res, link)
                parsed = parser(res, **kwargs)
                self.reducer(parsed, action.name, results)
                if self._is_parsable(res):
                    yield res
                else:
                    if action.static == False:
                        action.static = True
                if breaker(res, **kwargs):
                    break

        if action.static == True:
            yield parent_response

    def _handle_action(self, action, parent_response=None, results=None, **kwargs):
        if parent_response is None:
            parent_response = self.get(self.ROOT_URL, refresh=True)
            self._set_response_meta(parent_response, parent_response.url,)
            self._set_history(self.ROOT_URL, None, 'root')

        if isinstance(action, (Root, Url)):
            if isinstance(action, Root):
                if action.url == '/':
                    links = self.ROOT_URL
                else:
                    links = action.url
            elif isinstance(action, Url):
                generator = self._get_method(action.generator, 'generator')
                links = []
                if action.previewer:
                    lnks = []
                    previewer = self._get_method(action.previewer, 'previewer')
                    prev_urls = previewer(
                        parent_response, action.url, **kwargs)
                    prev_responses = self._visited_links(prev_urls)
                    for prev_res in prev_responses:
                        lnk = generator(prev_res, action.url, **kwargs)
                        links.append(lnk)
                    links = chain(*links)
                links = generator(parent_response, action.url, **kwargs)
            responses = self._visited_links(
                links=links,
                parent_response=parent_response,
                action=action,
                results=results,
                **kwargs
            )
        else:
            responses = self._crawl_link_pattern(
                parent_response=parent_response,
                action=action,
                results=results,
                **kwargs
            )
        return responses

    def get(self, link, **kwargs):
        url = abs_path(self.ROOT_URL, link)
        response = self._get(url, **kwargs)
        return response

    def post(self, url, **kwargs):
        response = self._post(url, **kwargs)
        return response

    def login(self, login_data):
        url = self.LOGIN_URL or self.ROOT_URL
        r = self.post(url, data=login_data, refresh=True)

    def scrap(self, until=None, _response=None, _relay=None, _results=None, **kwargs):
        _results = _results or defaultdict(list)
        action, *rest = _relay or self.LINK_RELAY
        responses = self._handle_action(
            action=action,
            parent_response=_response,
            results=_results,
            **kwargs
        )
        for response in responses:
            if until is not None and until == action.name:
                return
            if rest:
                self.scrap(
                    until=until,
                    _response=response,
                    _relay=rest,
                    _results=_results,
                    **kwargs
                )
        return _results

    def reducer(self, parsed, name, results):
        if parsed is None:
            return
        if isinstance(parsed, (str, bytes)):
            results[name].append(parsed)
        elif isinstance(parsed, abc.Mapping):
            results[name].append(parsed)
        elif isinstance(parsed, abc.Iterable):
            results[name].extend(list(parsed))

    def get_action(self, name, many=False):
        actions = []
        for action in self.LINK_RELAY:
            if action.name == name:
                if many == False:
                    return action
                else:
                    actions.append(action)
        if actions:
            return actions
        raise ValueError('Invalid name: {}'.format(name))

    def main_filter(self, link, query, match, **kwargs):
        return True

    def default_parser(self, response, **kwargs):
        return []

    def query_generator(self, parent_response, path, **kwargs):
        yield path

    def url_previewer(self, parent_response, path, **kwargs):
        yield path

    def crawl_breaker(self, response, **kwargs):
        return False
